@@ -1,8 +1,10 @@
-from .numba_functions import numba_expit, _compute_partial_integral, _array_LUT
+from .numba_functions import _array_LUT
 from .utils import (validate_estimation_options, irt_evaluation,
                     trim_response_set_and_counts,
                     convert_responses_to_kernel_sign,
-                    get_true_false_counts, mml_approx, create_beta_LUT)
+                    get_true_false_counts, mml_approx, 
+                    create_beta_LUT, _compute_partial_integral,
+                    tag_missing_data, INVALID_RESPONSE)
 from .polytomous_utils import condition_polytomous_response
 from .synthetic import (create_correlated_abilities,
                         create_synthetic_irt_dichotomous,
@@ -11,8 +13,7 @@ from .synthetic import (create_correlated_abilities,
 from .latent_ability_distribution import LatentPDF, CubicSplinePDF
 from .mml_methods import (rasch_mml, onepl_mml,
                           twopl_mml, grm_mml)
-from .mml_full_methods import (rasch_full, onepl_full, twopl_full,
-                               pcm_mml, gum_mml)
+from .mml_full_methods import pcm_mml, gum_mml
 from .mml_eap_methods import twopl_mml_eap, grm_mml_eap
 from .conditional_methods import rasch_conditional
 from .jml_methods import (rasch_jml, onepl_jml, twopl_jml,
