@@ -1,224 +1,15 @@
 import numpy as np
-from scipy import integrate, stats
-from scipy.optimize import fminbound, fmin_powell, fmin_slsqp
+from scipy.optimize import fmin_slsqp
 
-from girth import (irt_evaluation, convert_responses_to_kernel_sign,
-                   validate_estimation_options, mml_approx)
-from girth.utils import _get_quadrature_points
+from girth import validate_estimation_options
+from girth.utils import INVALID_RESPONSE 
 from girth.latent_ability_distribution import LatentPDF
-from girth.numba_functions import _compute_partial_integral
 from girth.polytomous_utils import (condition_polytomous_response,
                                     _credit_partial_integral,
                                     _unfold_partial_integral)
 from girth.ability_methods import _ability_eap_abstract
 
 
-def rasch_full(dataset, discrimination=1, options=None):
-    """ Estimates difficulty parameters in Rash IRT model.
-
-    Args:
-        dataset: [items x participants] matrix of True/False Values
-        discrimination: scalar of discrimination used in model (default to 1)
-        options: dictionary with updates to default options
-
-    Returns:
-        difficulty: (1d array) difficulty estimates
-
-    Options:
-        * max_iteration: int
-        * distribution: callable
-        * quadrature_bounds: (float, float)
-        * quadrature_n: int
-    """
-    return onepl_full(dataset, alpha=discrimination, options=options)
-
-
-def onepl_full(dataset, alpha=None, options=None):
-    """ Estimates parameters in an 1PL IRT Model.
-
-    This function is slow, please use onepl_mml
-
-    Args:
-        dataset: [items x participants] matrix of True/False Values
-        alpha: scalar of discrimination used in model (default to 1)
-        options: dictionary with updates to default options
-
-    Returns:
-        discrimination: (float) estimate of test discrimination
-        difficulty: (1d array) estimates of item diffiulties
-
-    Options:
-        * max_iteration: int
-        * distribution: callable
-        * quadrature_bounds: (float, float)
-        * quadrature_n: int
-
-    Notes:
-        If alpha is supplied then this solves a Rasch model
-    """
-    options = validate_estimation_options(options)
-    quad_start, quad_stop = options['quadrature_bounds']
-    quad_n = options['quadrature_n']
-
-    n_items = dataset.shape[0]
-    unique_sets, counts = np.unique(dataset, axis=1, return_counts=True)
-    the_sign = convert_responses_to_kernel_sign(unique_sets)
-
-    theta, weights = _get_quadrature_points(quad_n, quad_start, quad_stop)
-    distribution = options['distribution'](theta)
-    distribution_x_weights = distribution * weights
-
-    discrimination = np.ones((n_items,))
-    difficulty = np.zeros((n_items,))
-    the_output = np.zeros((the_sign.shape[1], theta.size), dtype='float64')
-
-    def alpha_min_func(alpha_estimate):
-        discrimination[:] = alpha_estimate
-
-        for iteration in range(options['max_iteration']):
-            previous_difficulty = difficulty.copy()
-
-            # Quadrature evaluation for values that do not change
-            partial_int = np.ones_like(the_output)
-            for ndx in range(n_items):
-                partial_int *= _compute_partial_integral(theta, difficulty[ndx], 
-                                                         discrimination[ndx], the_sign[ndx],
-                                                         the_output)
-            partial_int *= distribution_x_weights
-
-            for item_ndx in range(n_items):
-                # pylint: disable=cell-var-from-loop
-
-                # remove contribution from current item
-                local_int = _compute_partial_integral(theta, difficulty[item_ndx],
-                                                      discrimination[item_ndx],
-                                                      the_sign[item_ndx], the_output)
-
-                partial_int /= local_int
-
-                def min_local_func(beta_estimate):
-                    difficulty[item_ndx] = beta_estimate
-
-                    estimate_int = _compute_partial_integral(theta, difficulty[item_ndx],
-                                                             discrimination[item_ndx],
-                                                             the_sign[item_ndx], the_output)
-
-                    estimate_int *= partial_int
-                    otpt = np.sum(estimate_int, axis=1)
-                    return -np.log(otpt).dot(counts)
-
-                fminbound(min_local_func, -4, 4)
-
-                # Update the partial integral based on the new found values
-                estimate_int = _compute_partial_integral(theta, difficulty[item_ndx],
-                                                         discrimination[item_ndx],
-                                                         the_sign[item_ndx], the_output)
-                # update partial integral
-                partial_int *= estimate_int
-
-            if(np.abs(previous_difficulty - difficulty).max() < 1e-3):
-                break
-
-        cost = np.sum(partial_int, axis=1)
-        return -np.log(cost).dot(counts)
-
-    if alpha is None:  # OnePl Solver
-        alpha = fminbound(alpha_min_func, 0.1, 4)
-    else:  # Rasch Solver
-        alpha_min_func(alpha)
-
-    return {'Discrimination': alpha, 
-            'Difficulty': difficulty}
-
-
-def twopl_full(dataset, options=None):
-    """ Estimates parameters in a 2PL IRT model.
-
-    Please use twopl_mml instead.
-
-    Args:
-        dataset: [items x participants] matrix of True/False Values
-        options: dictionary with updates to default options
-
-    Returns:
-        discrimination: (1d array) estimates of item discrimination
-        difficulty: (1d array) estimates of item difficulties
-
-    Options:
-        * max_iteration: int
-        * distribution: callable
-        * quadrature_bounds: (float, float)
-        * quadrature_n: int
-"""
-    options = validate_estimation_options(options)
-    quad_start, quad_stop = options['quadrature_bounds']
-    quad_n = options['quadrature_n']
-
-    n_items = dataset.shape[0]
-    unique_sets, counts = np.unique(dataset, axis=1, return_counts=True)
-    the_sign = convert_responses_to_kernel_sign(unique_sets)
-
-    theta, weights = _get_quadrature_points(quad_n, quad_start, quad_stop)
-    distribution = options['distribution'](theta)
-    distribution_x_weights = distribution * weights
-
-    discrimination = np.ones((n_items,))
-    difficulty = np.zeros((n_items,))
-    the_output = np.zeros((the_sign.shape[1], theta.size), dtype='float64')
-
-    for iteration in range(options['max_iteration']):
-        previous_discrimination = discrimination.copy()
-
-        # Quadrature evaluation for values that do not change
-        partial_int = np.ones_like(the_output)
-        for ndx in range(n_items):
-            partial_int *= _compute_partial_integral(theta, difficulty[ndx], 
-                                                     discrimination[ndx], the_sign[ndx],
-                                                     the_output)
-        partial_int *= distribution_x_weights
-
-        for item_ndx in range(n_items):
-            # pylint: disable=cell-var-from-loop
-            local_int = _compute_partial_integral(theta, difficulty[item_ndx],
-                                                  discrimination[item_ndx],
-                                                  the_sign[item_ndx], the_output)
-
-            partial_int /= local_int
-
-            def min_func_local(estimate):
-                discrimination[item_ndx] = estimate[0]
-                difficulty[item_ndx] = estimate[1]
-
-                estimate_int = _compute_partial_integral(theta,
-                                                         difficulty[item_ndx],
-                                                         discrimination[item_ndx],
-                                                         the_sign[item_ndx], the_output)
-
-                estimate_int *= partial_int
-                otpt = np.sum(estimate_int, axis=1)
-
-                return -np.log(otpt).dot(counts)
-
-            # Two parameter solver that doesn't need derivatives
-            initial_guess = np.concatenate((discrimination[item_ndx, None],
-                                            difficulty[item_ndx, None]))
-            fmin_slsqp(min_func_local, initial_guess, disp=False,
-                       bounds=[(0.25, 4), (-4, 4)])
-
-            # Update the partial integral based on the new found values
-            estimate_int = _compute_partial_integral(theta, difficulty[item_ndx],
-                                                     discrimination[item_ndx],
-                                                     the_sign[item_ndx], the_output)
-            # update partial integral
-            partial_int *= estimate_int
-
-        if(np.abs(discrimination - previous_discrimination).max() < 1e-3):
-            break
-
-    return {'Discrimination': discrimination, 
-            'Difficulty': difficulty}
-
-
 def pcm_mml(dataset, options=None):
     """Estimate parameters for partial credit model.
 
@@ -243,8 +34,10 @@ def pcm_mml(dataset, options=None):
     """
     options = validate_estimation_options(options)
 
-    responses, item_counts = condition_polytomous_response(dataset, trim_ends=False,
-                                                           _reference=0.0)
+    cpr_result =  condition_polytomous_response(dataset, trim_ends=False, _reference=0.0)
+    responses, item_counts, valid_response_mask = cpr_result
+    invalid_response_mask = ~valid_response_mask
+
     n_items = responses.shape[0]
 
     # Quadrature Locations
@@ -262,6 +55,11 @@ def pcm_mml(dataset, options=None):
     for ndx in range(n_items):
         betas[ndx, 1:item_counts[ndx]] = np.linspace(-1, 1, item_counts[ndx]-1)
 
+    # Set invalid index to zero, this allows minimal
+    # changes for invalid data and it is corrected
+    # during integration
+    responses[invalid_response_mask] = 0
+
     #############
     # 1. Start the iteration loop
     # 2. Estimate Dicriminatin/Difficulty Jointly
@@ -279,7 +77,8 @@ def pcm_mml(dataset, options=None):
         for item_ndx in range(n_items):
             partial_int *= _credit_partial_integral(theta, betas[item_ndx],
                                                     discrimination[item_ndx],
-                                                    responses[item_ndx])
+                                                    responses[item_ndx],
+                                                    invalid_response_mask[item_ndx])
         # Estimate the distribution if requested
         distribution_x_weight = latent_pdf(partial_int, iteration)
         partial_int *= distribution_x_weight        
@@ -293,14 +92,16 @@ def pcm_mml(dataset, options=None):
             # Remove the previous output
             old_values = _credit_partial_integral(theta, previous_betas[item_ndx],
                                                   previous_discrimination[item_ndx],
-                                                  responses[item_ndx])
+                                                  responses[item_ndx],
+                                                  invalid_response_mask[item_ndx])
             partial_int /= old_values
 
             def _local_min_func(estimate):
                 new_betas[1:] = estimate[1:]
                 new_values = _credit_partial_integral(theta, new_betas,
                                                       estimate[0],
-                                                      responses[item_ndx])
+                                                      responses[item_ndx],
+                                                      invalid_response_mask[item_ndx])
                 new_values *= partial_int
                 otpt = np.sum(new_values, axis=1)
                 return -np.log(otpt).sum()
@@ -318,7 +119,8 @@ def pcm_mml(dataset, options=None):
 
             new_values = _credit_partial_integral(theta, betas[item_ndx],
                                                   discrimination[item_ndx],
-                                                  responses[item_ndx])
+                                                  responses[item_ndx],
+                                                  invalid_response_mask[item_ndx])
 
             partial_int *= new_values
 
@@ -330,7 +132,8 @@ def pcm_mml(dataset, options=None):
     for item_ndx in range(n_items):
         partial_int *= _credit_partial_integral(theta, betas[item_ndx],
                                                 discrimination[item_ndx],
-                                                responses[item_ndx])
+                                                responses[item_ndx],
+                                                invalid_response_mask[item_ndx])
 
     # TODO:  look where missing values are and place NAN there instead
     # of appending them to the end
@@ -382,8 +185,10 @@ def gum_mml(dataset, delta_sign=(0, 1), options=None):
     """
     options = validate_estimation_options(options)
 
-    responses, item_counts = condition_polytomous_response(dataset, trim_ends=False,
-                                                           _reference=0.0)
+    cpr_result = condition_polytomous_response(dataset, trim_ends=False, _reference=0.0)
+    responses, item_counts, valid_response_mask = cpr_result
+    invalid_response_mask = ~valid_response_mask
+
     n_items = responses.shape[0]
 
     # Interpolation Locations
@@ -410,6 +215,11 @@ def gum_mml(dataset, delta_sign=(0, 1), options=None):
     delta_ndx = delta_sign[0]
     delta_multiplier = np.sign(delta_sign[1])
 
+    # Set invalid index to zero, this allows minimal
+    # changes for invalid data and it is corrected
+    # during integration
+    responses[invalid_response_mask] = 0    
+
     #############
     # 1. Start the iteration loop
     # 2. Estimate Dicriminatin/Difficulty Jointly
@@ -430,7 +240,9 @@ def gum_mml(dataset, delta_sign=(0, 1), options=None):
                                                     betas[item_ndx],
                                                     discrimination[item_ndx],
                                                     fold_span[item_ndx],
-                                                    responses[item_ndx])
+                                                    responses[item_ndx],
+                                                    invalid_response_mask[item_ndx])
+
         # Estimate the distribution if requested
         distribution_x_weight = latent_pdf(partial_int, iteration)
         partial_int *= distribution_x_weight
@@ -445,7 +257,8 @@ def gum_mml(dataset, delta_sign=(0, 1), options=None):
                                                   previous_betas[item_ndx],
                                                   previous_discrimination[item_ndx],
                                                   fold_span[item_ndx],
-                                                  responses[item_ndx])
+                                                  responses[item_ndx],
+                                                  invalid_response_mask[item_ndx])
             partial_int /= old_values
             
             new_betas = np.full((betas.shape[1],), np.nan)
@@ -454,7 +267,8 @@ def gum_mml(dataset, delta_sign=(0, 1), options=None):
                 new_values = _unfold_partial_integral(theta, estimate[1],
                                                       new_betas,
                                                       estimate[0], fold_span[item_ndx],
-                                                      responses[item_ndx])
+                                                      responses[item_ndx], 
+                                                      invalid_response_mask[item_ndx])
 
                 new_values *= partial_int
                 otpt = np.sum(new_values, axis=1)
@@ -477,7 +291,8 @@ def gum_mml(dataset, delta_sign=(0, 1), options=None):
                                                   betas[item_ndx],
                                                   discrimination[item_ndx],
                                                   fold_span[item_ndx],
-                                                  responses[item_ndx])
+                                                  responses[item_ndx],
+                                                  invalid_response_mask[item_ndx])
 
             partial_int *= new_values
 
@@ -494,7 +309,8 @@ def gum_mml(dataset, delta_sign=(0, 1), options=None):
                                                 betas[item_ndx],
                                                 discrimination[item_ndx],
                                                 fold_span[item_ndx],
-                                                responses[item_ndx])    
+                                                responses[item_ndx],
+                                                invalid_response_mask[item_ndx])    
     # Compute statistics for final iteration
     null_metrics = latent_pdf.compute_metrics(partial_int, latent_pdf.null_distribution * 
                                               latent_pdf.weights, 0)
